local ptpeppino = "ptpeppino"

local function isValid(object)
	return object and object.valid
end

freeslot(
	-- Peppino object:
	-- Just a copy of thok but so we don't
	-- need to iterate through every thok.
	"MT_PEPPINOTAUNT_THOK",
	-- Taunt flash sprite:
	"SPR_PTTF",
	-- Taunt sprites:
	"SPR_TANT",
	-- Taunt frames:
	"S_PEPPINOCHAR_TAUNT1",
	"S_PEPPINOCHAR_TAUNT2",
	"S_PEPPINOCHAR_TAUNT3",
	"S_PEPPINOCHAR_TAUNT4", 
	"sfx_ptaunt") -- Taunt SFX.
	
mobjinfo[MT_PEPPINOTAUNT_THOK] = {
	spawnstate = S_THOK,
	spawnhealth = 1000,
	reactiontime = 8,
	speed = 8,
	radius = 32*FRACUNIT,
	height = 64*FRACUNIT,
	mass = 16,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY|MF_DONTENCOREMAP,
}
	
states[S_PEPPINOCHAR_TAUNT1] = {
	sprite = SPR_PTTF,
	frame = FF_FULLBRIGHT|A,
	tics = 1,
	action = A_PlaySound,
	var1 = sfx_ptaunt,
	var2 = (1<<16) + 1, -- Don't play during the preticker, play at actor's origin
	nextstate = S_PEPPINOCHAR_TAUNT2,
}

states[S_PEPPINOCHAR_TAUNT2] = {
	sprite = SPR_PTTF,
	frame = FF_FULLBRIGHT|C,
	tics = 2,
	nextstate = S_PEPPINOCHAR_TAUNT3,
}

states[S_PEPPINOCHAR_TAUNT3] = {
	sprite = SPR_PTTF,
	frame = FF_FULLBRIGHT|B,
	tics = 2,
	nextstate = S_PEPPINOCHAR_TAUNT4,
}

states[S_PEPPINOCHAR_TAUNT4] = {
	sprite = SPR_PTTF,
	frame = FF_FULLBRIGHT|C,
	tics = 4,
	nextstate = S_PEPPINOCHAR_TAUNT4,
}

local totalTauntDuration = 0
-- Maths to determine how long the taunt is.
for i = S_PEPPINOCHAR_TAUNT1, S_PEPPINOCHAR_TAUNT4 do
	totalTauntDuration = $ + states[i].tics
end
	
local function spawnTauntFlash(origin)
	local tauntFlash = P_SpawnMobjFromMobj(origin, 0, 0, 0, MT_PEPPINOTAUNT_THOK)
	tauntFlash.hitlag = 0
	tauntFlash.state = S_PEPPINOCHAR_TAUNT1
	tauntFlash.rollangle = origin.rollangle
	tauntFlash.roll = origin.roll
	tauntFlash.pitch = origin.pitch
	tauntFlash.tics = $ + 1
	tauntFlash.target = origin
	tauntFlash.dispoffset = -2
	return tauntFlash
end

local function spawnTauntFrame(origin)
	local tauntFrame = P_SpawnMobjFromMobj(origin, 0, 0, 0, MT_PEPPINOTAUNT_THOK)
	tauntFrame.hitlag = 0
	tauntFrame.color = origin.player.skincolor
	tauntFrame.colorized = origin.colorized
	tauntFrame.rollangle = origin.rollangle
	tauntFrame.roll = origin.roll
	tauntFrame.pitch = origin.pitch
	tauntFrame.sprite = SPR_TANT
	tauntFrame.target = origin
	tauntFrame.dispoffset = 1
	tauntFrame.tics = INT32_MAX
	tauntFrame.frame = FF_FULLBRIGHT|P_RandomRange(0, 8)
	return tauntFrame
end

local function taunt(player)
	local pMo = player.mo
	spawnTauntFlash(pMo)
	spawnTauntFrame(pMo)
	pMo.sprite = SPR_NULL
end

local function canTaunt(player)
	-- Are you even real?
	if not (isValid(player) and not player.spectator and isValid(player.mo)) then return false end
	if player.mo.skin ~= ptpeppino then return false end -- Are you peppino?
	if player.spinouttimer > 0 then return false end -- Are you hurt? Can't taunt while you're dealt damage.
	if player.markedfordeath then return false end -- You WILL die. Don't taunt.
	if player.tumblebounces > 0 then return false end -- Stumble is a damage state, don't taunt.
	if (player.pflags & PF_FAULT) == PF_FAULT then return false end -- You faulted in POSITION, lol.
	if (player.pflags & PF_ELIMINATED) == PF_ELIMINATED then return false end -- Done and dusted in battle.
	if (player.pflags & PF_NOCONTEST) == PF_NOCONTEST then return false end -- No contest explosion.
	if (player.pflags & PF_VOID) == PF_VOID	then return false end -- Got sent to the shadow realm (latejoin, position).
	if player.mo.health <= 0 then return false end -- No health, you're dead. Don't taunt.
	
	return true
end

--[[
COM_AddCommand("peppinotaunt", function(player)
	if not canTaunt(player) then return end
	if player.mo.hitlag > 0 then return end
	player.lasthitlag = totalTauntDuration
	player.mo.hitlag = totalTauntDuration
	taunt(player)
end)

COM_AddCommand("kms", function(player)
    P_DamageMobj(player.mo, nil, nil, 9999, DMG_INSTAKILL)
end)
]]

addHook("PlayerThink", function(player)
	-- Can you taunt?
	if not canTaunt(player) then return end
	
	-- Init this custom var with something.
	if not player.lasthitlag then player.lasthitlag = 0 end
	
	-- If we just got inflicted hitlag and it is equal or higher than the taunt duration...
	if player.lasthitlag == 0 and player.mo.hitlag >= totalTauntDuration then
		taunt(player)
	end
	
	player.lasthitlag = player.mo.hitlag
end)

addHook("MobjThinker", function(mobj)
	if not (isValid(mobj) and isValid(mobj.target) and mobj.target.hitlag > 0) then
		mobj.target.sprite = SPR_PLAY
		P_KillMobj(mobj)
	end
end, MT_PEPPINOTAUNT_THOK)
